import requests
import re
import urllib3
import html
from typing import Optional, Dict, Any


class SBVBankInfoAPI:
    BASE_URL = "https://www.sbv.gov.vn/application-frontend/faces/welcome.jspx"
    
    def __init__(self):
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self.session = requests.Session()
        self.session.verify = False
        self.session.allow_redirects = False
        
        self.frontend_value = None
        self.afrloop_value = None
        self.ctrl_state = None
        self.viewstate = None
    
    def _get_base_headers(self, referer: Optional[str] = None) -> Dict[str, str]:
        headers = {
            'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/136.0.0.0 Safari/537.36",
            'Accept': "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            'Accept-Encoding': "gzip, deflate, br, zstd",
            'sec-ch-ua': '"Chromium";v="136", "Google Chrome";v="136", "Not.A/Brand";v="99"',
            'sec-ch-ua-mobile': "?0",
            'sec-ch-ua-platform': '"Windows"',
            'Upgrade-Insecure-Requests': "1",
            'Accept-Language': "vi"
        }
        
        if referer:
            headers['Referer'] = referer
            headers['Sec-Fetch-Site'] = "same-origin"
        else:
            headers['Sec-Fetch-Site'] = "none"
            
        return headers
    
    def _extract_session_values(self, text: str) -> tuple:
        frontend_match = re.search(r'frontend=([^";\'\s]+)', text)
        afrloop_match = re.search(r'"_afrLoop",\s*"([^"]+)"', text)
        
        if not frontend_match or not afrloop_match:
            raise Exception("Không thể lấy được session values")
            
        return frontend_match.group(1).strip("'\""), afrloop_match.group(1)
    
    def _extract_viewstate(self, text: str) -> Optional[str]:
        viewstate_pattern = r'<input type="hidden" name="javax\.faces\.ViewState" value="([^"]+)"'
        match = re.search(viewstate_pattern, text)
        return match.group(1) if match else None
    
    def _extract_url_params(self, url: str) -> tuple:
        afr_loop_match = re.search(r'_afrLoop=([^&]+)', url)
        ctrl_state_match = re.search(r'_adf\.ctrl-state=([^&]+)', url)
        
        return (
            afr_loop_match.group(1) if afr_loop_match else None,
            ctrl_state_match.group(1) if ctrl_state_match else None
        )
    
    def _initialize_session(self) -> None:
        headers = self._get_base_headers()
        headers.update({
            'Sec-Fetch-Mode': "navigate",
            'Sec-Fetch-User': "?1",
            'Sec-Fetch-Dest': "document"
        })
        
        response = self.session.get(self.BASE_URL, headers=headers)
        self.frontend_value, self.afrloop_value = self._extract_session_values(response.text)
    
    def _get_authenticated_page(self) -> None:
        url = f"{self.BASE_URL};frontend={self.frontend_value}?_afrLoop={self.afrloop_value}&_afrWindowMode=0&_afrWindowId=null"
        headers = self._get_base_headers(self.BASE_URL)
        headers.update({
            'Sec-Fetch-Mode': "navigate",
            'Sec-Fetch-Dest': "document"
        })
        
        response = self.session.get(url, headers=headers, allow_redirects=True)
        self.viewstate = self._extract_viewstate(response.text)
        self.afrloop_value, self.ctrl_state = self._extract_url_params(response.url)
    
    def _navigate_to_search_page(self) -> None:
        url = f"{self.BASE_URL}?_adf.ctrl-state={self.ctrl_state}"
        headers = self._get_base_headers(
            f"{self.BASE_URL}?_afrLoop=14537936240771943&_afrWindowMode=0&_adf.ctrl-state={self.ctrl_state}"
        )
        headers.update({
            'Host': "www.sbv.gov.vn",
            'Connection': "keep-alive",
            'Cache-Control': "max-age=0",
            'Origin': "https://www.sbv.gov.vn",
            'Sec-Fetch-Mode': "navigate",
            'Sec-Fetch-User': "?1",
            'Sec-Fetch-Dest': "document"
        })
        
        data = {
            "org.apache.myfaces.trinidad.faces.FORM": "f1",
            "javax.faces.ViewState": self.viewstate,
            "event": "pt1:pt_i1:4:pt_i2:5:j_id__ctru61pc3:0:pt_cl7",
            "event.pt1:pt_i1:4:pt_i2:5:j_id__ctru61pc3:0:pt_cl7": '<m xmlns="http://oracle.com/richClient/comm"><k v="type"><s>action</s></k></m>'
        }
        
        response = self.session.post(url, headers=headers, data=data, allow_redirects=True)
        afrloop_match = re.search(r'"_afrLoop",\s*"([^"]+)"', response.text)
        if afrloop_match:
            self.afrloop_value = afrloop_match.group(1)
    
    def _prepare_search_form(self) -> None:
        params = {
            '_afrLoop': self.afrloop_value,
            '_afrWindowMode': "0",
            '_adf.ctrl-state': self.ctrl_state + "3"
        }
        
        headers = self._get_base_headers(self.BASE_URL)
        headers.update({
            'Sec-Fetch-Mode': "navigate",
            'Sec-Fetch-Dest': "document"
        })
        
        response = self.session.get(self.BASE_URL, params=params, headers=headers)
        self.viewstate = self._extract_viewstate(response.text)
        self.afrloop_value, self.ctrl_state = self._extract_url_params(response.url)
    
    def _search_bank_info(self, bank_code: str) -> requests.Response:
        params = {'_adf.ctrl-state': self.ctrl_state}
        
        payload = {
            'pt1:r1:0:s2:it4': bank_code,
            'pt1:r1:0:s2:it3': "",
            'pt1:r1:0:s2:id1': "",
            'pt1:r1:0:s2:id2': "",
            'pt1:r1:0:s2:soc5': "",
            'pt1:r1:0:s2:soc6': "",
            'pt1:r1:0:s2:soc7': "",
            'org.apache.myfaces.trinidad.faces.FORM': "f1",
            'javax.faces.ViewState': self.viewstate,
            'oracle.adf.view.rich.RENDER': "pt1:r1",
            'oracle.adf.view.rich.DELTAS': "{pt1:r1:0:pc2:t2={viewportSize=13},pt1:r1:0:pc2:_clmHdnT={_unconvertedValue=},pt1:r1:0:pc2:_clmHdnC={_unconvertedValue=}}",
            'event': "pt1:r1:0:s2:btn-search",
            'event.pt1:r1:0:s2:btn-search': "<m xmlns=\"http://oracle.com/richClient/comm\"><k v=\"type\"><s>action</s></k></m>",
            'oracle.adf.view.rich.PROCESS': "pt1:r1"
        }
        
        headers = self._get_base_headers(
            f"{self.BASE_URL}?_afrLoop=14539283850509943&_afrWindowMode=0&_adf.ctrl-state=hed6k8ljk_13"
        )
        headers.update({
            'Adf-Rich-Message': "true",
            'Origin': "https://www.sbv.gov.vn",
            'Sec-Fetch-Mode': "cors",
            'Sec-Fetch-Dest': "empty"
        })
        
        return self.session.post(self.BASE_URL, params=params, data=payload, headers=headers)
    
    def _parse_bank_info(self, response_text: str) -> Dict[str, str]:
        patterns = {
            'unit_name': r'<a id="pt1:r1:0:pc2:t2:0:cl2"[^>]*>([^<]+)</a>',
            'effective_date': r'<td style="width:10%"[^>]*><span style="white-space: nowrap"><span class="x2rf">(\d{2}/\d{2}/\d{4})</span></span></td>',
            'status': r'<td style="width:20%"[^>]*><span style="white-space: nowrap"><span class="x2rf"[^>]*>([^<]+)</span></span></td>'
        }
        
        result = {}
        
        for key, pattern in patterns.items():
            match = re.search(pattern, response_text)
            if match:
                value = match.group(1)
                if key in ['unit_name', 'status']:
                    value = html.unescape(value)
                result[key] = value
            else:
                result[key] = "Not found"
        
        return result
    
    def get_bank_info(self, bank_code: str) -> Dict[str, str]:
        try:
            self._initialize_session()
            self._get_authenticated_page()
            self._navigate_to_search_page()
            self._prepare_search_form()
            
            response = self._search_bank_info(bank_code)
            return self._parse_bank_info(response.text)
            
        except Exception as e:
            return {"error": str(e)}
    
    def display_bank_info(self, bank_code: str) -> None:
        info = self.get_bank_info(bank_code)
        
        if "error" in info:
            print(f"Lỗi: {info['error']}")
            return
        
        print(f"Tên đơn vị: {info.get('unit_name', 'Not found')}")
        print(f"Ngày hiệu lực: {info.get('effective_date', 'Not found')}")
        print(f"Trạng thái: {info.get('status', 'Not found')}")


if __name__ == "__main__":
    scraper = SBVBankInfoAPI()
    scraper.display_bank_info("01204050")